#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Captured metadata of NikonD70.jpg .
Make sure to ignore "Directory" and "FileModifyDate" when using this for
comparison purposes..
"""
__revision__ = "$Id$"

fields = {'AFAperture': u'4.5',
          'AFAreaMode': u'Single Area',
          'AFPoint': u'Center',
          'AFPointsInFocus': u'Center',
          'Aperture': u'9.0',
          #'AutoBracketRelease': u'None',
          'BitsPerSample': u'8',
          'BlueBalance': u'1.636719',
          'CFAPattern': u'[Blue,Green][Green,Red]',
          'CircleOfConfusion': u'0.020 mm',
          'ColorComponents': u'3',
          'ColorHue': u'Mode1a',
          'ColorSpace': u'Uncalibrated',
          'ComponentsConfiguration': u'Y, Cb, Cr, -',
          'CompressedBitsPerPixel': u'4',
          'Compression': u'JPEG (old-style)',
          'Contrast': u'Normal',
          'CreateDate': u'1080636226',
          'CustomRendered': u'Normal',
          'DOF': u'4.70 m (3.85 - 8.55)',
          'DateTimeOriginal': u'1080636226',
          'DigitalZoomRatio': u'1',
          'EffectiveMaxAperture': u'4.5',
          'EncodingProcess': u'Baseline DCT, Huffman coding',
          'ExifByteOrder': u'Big-endian (Motorola, MM)',
          'ExifVersion': u'0221',
          'ExitPupilPosition': u'89.0 mm',
          'ExposureBracketValue': u'0',
          'ExposureCompensation': u'0',
          'ExposureDifference': u'0',
          'ExposureMode': u'Auto',
          'ExposureProgram': u'Program AE',
          'ExposureTime': u'1/250',
          'ExternalFlashFirmware': u'n/a',
          'ExternalFlashFlags': u'(none)',
          'FNumber': u'9.0',
          'FOV': u'26.7 deg (2.52 m)',
          'Flash': u'No Flash',
          'FlashCommanderMode': u'Off',
          'FlashControlMode': u'Off',
          'FlashExposureBracketValue': u'0.0',
          'FlashExposureComp': u'0',
          'FlashGroupAControlMode': u'Off',
          #'FlashGroupAExposureComp': u'0',
          'FlashGroupBControlMode': u'Off',
          #'FlashGroupBExposureComp': u'0',
          'FlashInfoVersion': u'0100',
          'FlashMode': u'Did Not Fire',
          'FlashSetting': u'Normal',
          'FlashType': u'',
          'FlashpixVersion': u'0100',
          'FocalLength': u'50.0 mm',
          'FocalLength35efl': u'50.0 mm (35 mm equivalent: 75.0 mm)',
          'FocalLengthIn35mmFormat': u'75 mm',
          'FocusDistance': u'5.31 m',
          'FocusMode': u'AF-S',
          'FocusPosition': u'0xa1',
          'GainControl': u'None',
          'Gamma': u'2.2',
          'HueAdjustment': u'0',
          'HyperfocalDistance': u'13.87 m',
          'ISO': u'200',
          'ISOSetting': u'200',
          'ImageOptimization': u'Normal',
          'Lens': u'18-70mm f/3.5-4.5',
          'LensDataVersion': u'0101',
          'LensFStops': u'5.33',
          'LensID': u'AF-S DX Zoom-Nikkor 18-70mm f/3.5-4.5G IF-ED',
          'LensIDNumber': u'127',
          'LensSpec': u'18-70mm f/3.5-4.5 G',
          'LensType': u'G',
          'LightSource': u'Natural',
          'LightValue': u'13.3',
          'MCUVersion': u'132',
          'MIMEType': u'image/jpeg',
          'Make': u'NIKON CORPORATION',
          'MakerNoteVersion': u'2.10',
          'MaxApertureAtMaxFocal': u'4.5',
          'MaxApertureAtMinFocal': u'3.6',
          'MaxApertureValue': u'4.4',
          'MaxFocalLength': u'71.3 mm',
          'MeteringMode': u'Multi-segment',
          'MinFocalLength': u'18.3 mm',
          'Model': u'NIKON D70',
          'ModifyDate': u'1080636226',
          'NoiseReduction': u'Off',
          'PrimaryChromaticities': u'0.64 0.33 0.21 0.71 0.15 0.06',
          'ProgramShift': u'0',
          'Quality': u'Fine',
          'RedBalance': u'2.015625',
          'ResolutionUnit': u'inches',
          'Saturation': u'Normal',
          'ScaleFactor35efl': u'1.5',
          'SceneCaptureType': u'Standard',
          'SceneType': u'Directly photographed',
          'SensingMethod': u'One-chip color area',
          'SensorPixelSize': u'7.8 x 7.8 um',
          'SerialNumber': u'No= 100005e5',
          'Sharpness': u'Normal',
          'ShootingMode': u'Single-Frame',
          'ShutterCount': u'1193',
          'ShutterSpeed': u'1/250',
          'Software': u'Ver.1.01',
          'SubSecCreateDate': u'1080636226',
          'SubSecDateTimeOriginal': u'1080636226',
          'SubSecModifyDate': u'1080636226',
          'SubSecTime': u'00',
          'SubSecTimeDigitized': u'00',
          'SubSecTimeOriginal': u'00',
          'SubjectDistanceRange': u'Unknown',
          'ThumbnailImage':
                      u'(Binary data 8930 bytes, use -b option to extract)',
          'ThumbnailLength': u'8930',
          'ToneComp': u'Auto',
          'UserComment': u'',
          'VariProgram': u'',
          'WB_RGBGLevels': u'516 256 419 256',
          'WhiteBalance': u'Auto',
          'WhiteBalanceFineTune': u'0',
          'WhitePoint': u'0.313 0.329',
          'XResolution': u'300',
          'YResolution': u'300'}
